import EventService from '@/services/EventService.js'

export const namespaced = true

export const state = {
  events: [],
  eventsTotal: 0,
  event: {},
  perPage: 3
}

export const mutations = {
  ADD_EVENT(state, event) {
    state.events.push(event)
  },
  SET_EVENTS(state, events) {
    state.events = events
  },
  SET_EVENTS_TOTAL(state, eventsTotal) {
    state.eventsTotal = eventsTotal
  },
  SET_EVENT(state, event) {
    state.event = event
  }
}

export const actions = {
  createEvent({ commit, dispatch }, event) {
    return EventService.postEvent(event)
      .then(() => {
        commit('ADD_EVENT', event)
        const notification = {
          type: 'success',
          message: 'Your event has been created!'
        }
        dispatch('notification/add', notification, { root: true })
      })
      .catch(error => {
        const notification = {
          type: 'error',
          message: 'There was a problem creating your event: ' + error.message
        }
        dispatch('notification/add', notification, { root: true })
        throw error
      })
  },
  // perPage is removed from the payload because it is now part of the global state storage
  fetchEvents({ commit, dispatch, state }, { page }) {
    return EventService.getEvents(state.perPage, page) // returning the promise
      .then(response => {
        commit('SET_EVENTS_TOTAL', parseInt(response.headers['x-total-count']))
        commit('SET_EVENTS', response.data)
      })
      .catch(error => {
        const notification = {
          type: 'error',
          message: 'There was a problem fetching events: ' + error.message
        }
        dispatch('notification/add', notification, { root: true })
      })
  },
  fetchEvent({ commit, getters }, id) {
    // dispatch was removed as a param because we cut out the catch error block below.
    var event = getters.getEventById(id)

    if (event) {
      commit('SET_EVENT', event)
      return event // added to return the event to our beforeEnter Per route gaurd
    } else {
      return EventService.getEvent(id) // we needed to add return so that the promise is returned to the ShowEvents beforeRouteEnter hook
        .then(response => {
          commit('SET_EVENT', response.data)
          return response.data
        })
      // This was removed so that we could 404 any events that didn't exist. We want to catch the error in the beforeEnter hook instead
      // .catch(error => {
      //   const notification = {
      //     type: 'error',
      //     message: 'There was a problem fetching event: ' + error.message
      //   }
      //   dispatch('notification/add', notification, { root: true })
      // })
    }
  }
}
export const getters = {
  getEventById: state => id => {
    return state.events.find(event => event.id === id)
  }
}
